



# https://askubuntu.com/questions/430194/no-such-file-or-directory-when-executing-usr-bin-time
# http://www.linuxintheshell.com/2013/02/26/phobia-2016-turkce-altyazili-hd-izle-filmi-izle-full-izle/

# too assess memory:
# Empirtical Approach to run the tool with more and more data
# https://researchcomputing.princeton.edu/support/knowledge-base/memory
#   - collect all reports
#   - parse the Memorz Utilized...
# e.g. using seff

# Or dynamically get the currently running JOBID:
#   - by running series of squeue -U franckl
#   - by writing for this assessment work all the JOBID/<hostname> in a file
# -> then ssh <hostname> and htop -u $USER